@@ -63,6 +63,8 @@
 #define PAGESZ	4096
 #define DLS_TYPE "dls"
 #define DLSSZ	512
+#define RAW_TYPE "raw"
+#define RAWSZ 	512
 
 void 
 usage(char *myProg)  
@@ -81,6 +83,7 @@ usage(char *myProg)
 	fs_print_types();
 	printf("\t%s\n", DLS_TYPE);
 	printf("\t%s\n", SWAP_TYPE);
+	printf("\t%s\n", RAW_TYPE);
 
 	exit(1);
 }
@@ -145,8 +148,9 @@ main(int argc, char **argv)
 			usage(argv[0]);
 
 		if ((strcmp(SWAP_TYPE, fstype) == 0) ||
-		  (strcmp(DLS_TYPE, fstype) == 0) ) {
-			printf("swap type not supported for stat option\n");
+		  (strcmp(DLS_TYPE, fstype) == 0) ||
+		  (strcmp(RAW_TYPE, fstype) == 0) ) {
+			printf("swap/raw/dls type not supported for stat option\n");
 			usage(argv[0]);
 		}
 
@@ -166,6 +170,7 @@ main(int argc, char **argv)
 
 	/* open appropriate file */
 	if ((strcmp(SWAP_TYPE, fstype) == 0) ||
+	  (strcmp(RAW_TYPE, fstype) == 0) ||
 	  (strcmp(DLS_TYPE, fstype) == 0)) {
 		if (NULL == (hSwap = fopen(argv[optind++], "r")))
 			error("error opening: %s\n", argv[optind-1]);
@@ -175,7 +180,7 @@ main(int argc, char **argv)
 	}
 
 	/* we already checked above if we had STAT and SWAP, so we don't
-	 * nee to check again 
+	 * need to check again 
 	 */
 	if (format & STAT) {
 		stats(fs);
@@ -194,6 +199,9 @@ main(int argc, char **argv)
 		else if (strcmp(DLS_TYPE, fstype) == 0) {
 			size = DLSSZ;
 		}
+		else if (strcmp(RAW_TYPE, fstype) == 0) {
+			size = RAWSZ;
+		}
 		else {
 			size = fs->block_size;
 		}
@@ -211,6 +219,7 @@ main(int argc, char **argv)
 
 	/* non swap */
 	if ((strcmp(SWAP_TYPE, fstype) != 0) &&
+	  (strcmp(RAW_TYPE, fstype) != 0) && 
 	  (strcmp(DLS_TYPE, fstype) != 0)) {
 		if (size < 0) { 
 			error("Invalid size: %i\n", size);
@@ -224,6 +233,7 @@ main(int argc, char **argv)
 
 	/* Read the data */
 	if ((strcmp(SWAP_TYPE, fstype) == 0) ||
+	  (strcmp(RAW_TYPE, fstype) == 0) ||
 	  (strcmp(DLS_TYPE, fstype) == 0)) {
 		if(fseek(hSwap, size*block, SEEK_SET)) 
 			error ("error seeking to: %d\n", size*block);
@@ -336,6 +346,7 @@ main(int argc, char **argv)
 	fs_buf_free(buf);
 
 	if ((strcmp(SWAP_TYPE, fstype) == 0) ||
+	  (strcmp(RAW_TYPE, fstype) == 0) ||
 	  (strcmp(DLS_TYPE, fstype) == 0)) 
 		fclose(hSwap);
 	else 
