@@ -200,7 +200,7 @@ slack_inode_act(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 	flen = fs_inode->size;
 	if ((fs->ftype & FSMASK) == FATFS_TYPE) {
 		fs->file_walk(fs, fs_inode, 0, 0, 
-		  (FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_SLACK),
+		  (FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK),
 		  slack_file_act, ptr);
 	}
 	else {
@@ -211,7 +211,7 @@ slack_inode_act(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 
 			if (fs_data->flags & FS_DATA_NONRES) {
 				fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id, 
-				  (FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_SLACK),
+				  (FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK),
 				  slack_file_act, ptr);
 			}
 
