@@ -193,11 +193,13 @@ ext2fs_dinode_lookup(EXT2FS_INFO *ext2fs, INUM_T inum)
     ext2fs->inum = inum;
     if (verbose)
 		printf(
-		  "%lu m/l/s=%o/%d/%lu u/g=%d/%d macd=%lu/%lu/%lu/%lu\n",
+		  "%lu m/l/s=%o/%d/%llu u/g=%d/%d macd=%lu/%lu/%lu/%lu\n",
 		  (ULONG) inum, 
 		  getu16(fs, dino->i_mode),
 		  getu16(fs, dino->i_nlink),
-		  (ULONG) getu32(fs, dino->i_size),
+		  (ULLONG) (getu32(fs, dino->i_size) + 
+		    (getu16(fs, dino->i_mode) & EXT2_IN_REG) ?
+		    (u_int64_t) getu32(fs, dino->i_size_high) << 32 : 0),
 		  getu16(fs, dino->i_uid),
 		  getu16(fs, dino->i_gid),
 		  (ULONG) getu32(fs, dino->i_mtime),
@@ -211,16 +213,32 @@ ext2fs_dinode_lookup(EXT2FS_INFO *ext2fs, INUM_T inum)
 static void 
 ext2fs_copy_inode(EXT2FS_INFO *ext2fs, FS_INODE *fs_inode)
 {
-    int     i;
+	int     i;
 	ext2fs_inode 	*in = ext2fs->dinode;
 	FS_INFO *fs = (FS_INFO *)&ext2fs->fs_info;
-    GRPNUM_T grpnum;
-    UCHAR  *imap = 0;
-    INUM_T  ibase = 0;
+	ext2fs_sb *sb = ext2fs->fs;
+	GRPNUM_T grpnum;
+	UCHAR  *imap = 0;
+	INUM_T  ibase = 0;
 
 	fs_inode->mode =  getu16(fs, in->i_mode);
 	fs_inode->nlink =  getu16(fs, in->i_nlink);
+
 	fs_inode->size =  getu32(fs, in->i_size);
+
+	/* the general size value in the inode is only 32-bits,
+	 * but the i_dir_acl value is used for regular files to 
+	 * hold the upper 32-bits 
+	 *
+	 * The RO_COMPAT_LARGE_FILE flag in the super block will identify
+	 * if there are any large files in the file system
+	 */
+	if ((fs_inode->mode & EXT2_IN_REG) && 
+	  (getu32(fs, sb->s_feature_ro_compat) & 
+		  EXT2FS_FEATURE_RO_COMPAT_LARGE_FILE) ) {
+		fs_inode->size +=  ((u_int64_t)getu32(fs, in->i_size_high) << 32);
+	}
+
 	fs_inode->uid =  getu16(fs, in->i_uid);
 	fs_inode->gid =  getu16(fs, in->i_gid);
 	fs_inode->mtime =  getu32(fs, in->i_mtime);
@@ -561,7 +579,7 @@ ext2fs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 
 static int 
 ext2fs_file_walk_direct(FS_INFO *fs, FS_BUF *buf[],
-  OFF_T length, DADDR_T addr, int flags, FS_FILE_WALK_FN action, char *ptr)
+  size_t length, DADDR_T addr, int flags, FS_FILE_WALK_FN action, char *ptr)
 {
     int     read_count;
 	int 	myflags;
@@ -573,8 +591,11 @@ ext2fs_file_walk_direct(FS_INFO *fs, FS_BUF *buf[],
 
 	if (addr > fs->last_block) {
 		if (flags & FS_FLAG_NOABORT)  {
-			fprintf (stderr, "Invalid direct block address (too large): %lu",
-		  	  (ULONG) addr);
+			if (verbose) {
+				fprintf (logfp, 
+				  "Invalid direct block address (too large): %lu",
+		  		  (ULONG) addr);
+			}
 			return 0;
 		}
 		else {
@@ -614,11 +635,11 @@ ext2fs_file_walk_direct(FS_INFO *fs, FS_BUF *buf[],
 
 /* ext2fs_file_walk_indir - copy indirect block */
 static int 
-ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], OFF_T length,
+ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
   DADDR_T addr, int level, int flags, FS_FILE_WALK_FN action, char *ptr)
 {    
     char   *myname = "ext2fs_file_walk_indir";
-    OFF_T   todo_count = length;
+    size_t   todo_count = length;
     DADDR_T *iaddr;
     int     n;
     
@@ -627,8 +648,11 @@ ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], OFF_T length,
     
     if (addr > fs->last_block) {
         if (flags & FS_FLAG_NOABORT)  {
-        	fprintf (stderr, "Invalid indirect block address (too large): %lu",
-			  (ULONG)addr);
+			if (verbose) {
+				fprintf (logfp, 
+        		  "Invalid indirect block address (too large): %lu",
+				  (ULONG)addr);
+			}
             return 0;    
 		}
 		else {
@@ -661,8 +685,11 @@ ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], OFF_T length,
 
 	    if (getu32(fs, (u_int8_t *)&iaddr[n]) > fs->last_block) {
 			if (flags & FS_FLAG_NOABORT)  {
-				fprintf (stderr, "Invalid address in indirect list (too large): %lu",
-          		  (ULONG)getu32(fs, (u_int8_t *)&iaddr[n]) );
+				if (verbose) {
+					fprintf (logfp, 
+					  "Invalid address in indirect list (too large): %lu",
+     	     		  (ULONG)getu32(fs, (u_int8_t *)&iaddr[n]) );
+				}
 				return 0;    
 			}
 			else {
@@ -699,7 +726,7 @@ void
 ext2fs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
     int flags, FS_FILE_WALK_FN action, char *ptr)
 {
-    OFF_T   length;
+    size_t   length;
     FS_BUF **buf;
     int     level, retval, n;
 
@@ -867,7 +894,7 @@ ext2fs_fsstat(FS_INFO *fs, FILE *hFile)
 			fprintf(hFile, "Sparse Super, ");
 		if (getu32(fs, sb->s_feature_ro_compat) & 
 		  EXT2FS_FEATURE_RO_COMPAT_LARGE_FILE)
-			fprintf(hFile, "Large File, ");
+			fprintf(hFile, "Has Large Files, ");
 		if (getu32(fs, sb->s_feature_ro_compat) & 
 		  EXT2FS_FEATURE_RO_COMPAT_BTREE_DIR)
 			fprintf(hFile, "Btree Dir");
@@ -1105,7 +1132,8 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 		
 		fprintf(hFile, "\n");
 	}
-    fprintf(hFile, "size: %lu\n", (ULONG) fs_inode->size);
+
+    fprintf(hFile, "size: %llu\n", (ULLONG) fs_inode->size);
     fprintf(hFile, "num of links: %lu\n", (ULONG) fs_inode->nlink);
 
     if (sec_skew != 0) {
@@ -1293,11 +1321,11 @@ ext2fs_open(const char *name, unsigned char ftype)
 	/* allocate buffers */
 
 	/* inode map */
-    ext2fs->inode_map = mymalloc(fs->block_size);
+    ext2fs->inode_map = (unsigned char *) mymalloc(fs->block_size);
     ext2fs->imap_num = -1;
 
 	/* block map */
-    ext2fs->block_map = mymalloc(fs->block_size);
+    ext2fs->block_map = (unsigned char *) mymalloc(fs->block_size);
     ext2fs->bmap_num = -1;
 
 	/* dinode */
