@@ -358,7 +358,8 @@ ext2fs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
     dirleft = size;
     
     fs->file_walk(fs, fs_inode, 0, 0,
-      FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_SLACK, ext2fs_dent_action, "");
+      FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK, 
+	  ext2fs_dent_action, "");
 
     curdirptr = NULL;
 	dirptr = dirbuf;
