@@ -5,7 +5,7 @@
 ** Content and meta data layer support for the FAT file system 
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -19,8 +19,6 @@
 #include "mymalloc.h"
 #include "error.h"
 
-static int32_t	secwest;	/* Number of seconds west of UTC for time
-							* conversions */
 
 /*
  * Implementation NOTES 
@@ -65,7 +63,7 @@ getFAT(FATFS_INFO *fatfs, u_int32_t clust)
 	switch (fatfs->fs_info.ftype) {
 	  case MS12_FAT:
 		if (clust & 0xf000)
-			error("getFAT: FAT12 Cluster too large ");
+			error("getFAT: FAT12 Cluster %d too large", clust);
 
 		/* id the sector in the FAT */
 		sect = fatfs->firstfatsect + (clust + clust / 2) / fatfs->ssize;
@@ -106,7 +104,8 @@ getFAT(FATFS_INFO *fatfs, u_int32_t clust)
 		/* sanity check */
 		if ((retval > (fatfs->clustcnt + 1)) &&
 		  (retval < (0x0ffffff7 & FATFS_12_MASK)))
-			error ("getFAT: return FAT12 cluster too large");
+			error ("getFAT: return FAT12 cluster (%d) too large (%d)", 
+			  clust, retval);
 
 		break;
 
@@ -332,98 +331,36 @@ fatfs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 ** UNIX stores the time in seconds from 1970 in UTC
 ** FAT dates are the actual date with the year relative to 1980
 ** 
-** Modified version from OpenBSD source in:
-** src/sys/msdosfs/msdosfs_conv.c by Paul Popelka 
 */
 static int
 dos2unixtime(u_int16_t date, u_int16_t time)
 {
-    u_long seconds, lastseconds;
-    u_long m, month;
-    u_long y, year;
-    u_long days;
-    u_short *months;
-
-	/* Days in each month in a regular year */
-	u_short regyear[] = {
-		31, 28, 31, 30, 31, 30,
-		31, 31, 30, 31, 30, 31
-	};
-           
-	/* Days in each month in a leap year */
-	u_short leapyear[] = {  
-		31, 29, 31, 30, 31, 30,
-		31, 31, 30, 31, 30, 31
-	};
+	struct tm tm1;
+	int ret;
 
     if (date == 0) 
         return 0;
 
-	/* number of seconds in the last day */
-    seconds = ((time & FATFS_SEC_MASK) >> FATFS_SEC_SHIFT) * 2
-        + ((time & FATFS_MIN_MASK) >> FATFS_MIN_SHIFT) * 60
-        + ((time & FATFS_HOUR_MASK) >> FATFS_HOUR_SHIFT) * 3600;
+	tm1.tm_sec = ((time & FATFS_SEC_MASK) >> FATFS_SEC_SHIFT) * 2;
+	tm1.tm_min = ((time & FATFS_MIN_MASK) >> FATFS_MIN_SHIFT);
+	tm1.tm_hour = ((time & FATFS_HOUR_MASK) >> FATFS_HOUR_SHIFT);
 
-	/* Number of days, based on years */
-	days = 0;
-	year = (date & FATFS_YEAR_MASK) >> FATFS_YEAR_SHIFT;
-	for (y = 0; y < year; y++)
-		days += y & 0x03 ? 365 : 366;
+	tm1.tm_mday = ((date & FATFS_DAY_MASK) >> FATFS_DAY_SHIFT);
+	tm1.tm_mon = ((date & FATFS_MON_MASK) >> FATFS_MON_SHIFT) - 1;
+	tm1.tm_year = ((date & FATFS_YEAR_MASK) >> FATFS_YEAR_SHIFT) + 80;
 
-	/* Is the current year a leap year? */
-	months = year & 0x03 ? regyear : leapyear;
+	/* set the daylight savings variable to -1 so that mktime() figures
+	 * it out */
+	tm1.tm_isdst = -1;
 
-	/*
-	 * Prevent going from 0 to 0xffffffff in the following
-	 * loop.
-	 */
-	month = (date & FATFS_MON_MASK) >> FATFS_MON_SHIFT;
-	if (month == 0) {
-		printf("dos2unixtime(): month value out of range (%ld)\n",
-			month);
-		month = 1;
-	}
+	ret = mktime (&tm1);
 
-	/* days based on number of months so far this year */
-	for (m = 0; m < month - 1; m++)
-		days += months[m];
-	days += ((date & FATFS_DAY_MASK) >> FATFS_DAY_SHIFT) - 1;
-
-#define SECONDSTO1980   (((8 * 365) + (2 * 366)) * (24 * 60 * 60))
-
-	lastseconds = (days * 24 * 60 * 60) + SECONDSTO1980;
-
-	/* add up everything
-	 * we need the timezone data because FAT gives the exact time and
-	 * has no notion of timezones.  So, we must manually figure it out
-	 */
-    return seconds + lastseconds + secwest;
-         /* -+ daylight savings time correction */ ;
+	if (-1 == ret) 
+		error ("dos2unixtime: Error running mktime()");
 
+	return ret;
 }
 
-static void 
-set_secwest(void)
-{
-    struct  tm *tmptr;
-    time_t t;
-
-    t = time(NULL);
-    tmptr = localtime(&t);
-
-#if HAVE_TIMEZONE
-#if defined (CYGWIN)
-    secwest = _timezone;
-#else
-    secwest = timezone;
-#endif
-	/* account for daylight savings */
-    if (tmptr->tm_isdst)
-        secwest -= 3600;
-#else
-    secwest = -1 * tmptr->tm_gmtoff;
-#endif
-}
 
 
 /* 
@@ -1523,9 +1460,5 @@ fatfs_open(const char *name, unsigned char ftype)
 	fs->istat = fatfs_istat;
     fs->close = fatfs_close;
 
-
-	/* Figure out timezone difference for time conversion */
-	set_secwest();
-
 	return (fs);
 }
