@@ -563,7 +563,8 @@ fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
   
 	/* save the directory contents into dirbuf */
     fs->file_walk(fs, fs_inode, 0, 0,
-      FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_SLACK, fatfs_dent_action, "");
+      FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK, 
+	  fatfs_dent_action, "");
 
 	addrmax = addridx;
 	/* the pointer could have moved due to a realloc */
