@@ -181,6 +181,7 @@ main(int argc, char **argv)
 		return 1;
 	}
 
+	found = 0;
 
 	/* Since we start the walk on the root inode, then this will not show
 	** up in the above functions, so do it now
@@ -188,13 +189,13 @@ main(int argc, char **argv)
 	if (inode == fs->root_inum) {
 		if (flags & FS_FLAG_ALLOC)  {
 			printf("/\n");
+			found = 1;
 
 			if (!(localflags & FIND_ALL)) 
 				return 0;
 		}
 	}
 
-	found = 0;
 
 	if ((fs->ftype & FSMASK) == NTFS_TYPE) {
 		ntfs_find_file(fs, inode, type, id, flags, find_file, (char *)0);
@@ -203,7 +204,6 @@ main(int argc, char **argv)
 		fs->dent_walk(fs, fs->root_inum, flags, find_file, (char *)0);
 	}
 
-
 	if (!found) {
 
 		/* With FAT, we can at least give the name of the file and call
@@ -211,9 +211,11 @@ main(int argc, char **argv)
 		 */
 		if ((fs->ftype & FSMASK) == FATFS_TYPE) {
 			FS_INODE *fs_inode = fs->inode_lookup(fs, inode);
-			if (fs_inode->flags & FS_FLAG_UNALLOC)
-				printf("* ");
-			printf ("%s/%s\n", ORPHAN_STR, fs_inode->name->name);
+			if (fs_inode->name != NULL) {
+				if (fs_inode->flags & FS_FLAG_UNALLOC)
+					printf("* ");
+				printf ("%s/%s\n", ORPHAN_STR, fs_inode->name->name);
+			}
 		}
 		else {
 			printf("inode not currently used\n");
