@@ -98,12 +98,14 @@ void    fs_read_block(FS_INFO *fs, FS_BUF *buf, int len, DADDR_T addr,
 		if (LSEEK(fs->fd, offs, SEEK_SET) != offs)
 			error("read block lseek error (%llu): %m (%s)", 
 			  (ULLONG) offs, comment);
+
+    	fs->seek_pos = offs;
 	}
     if (read(fs->fd, buf->data, len) != len)
 		error("read block read error (%d@%llu): %m (%s)", 
 		  len, (ULLONG) offs, comment);
 #endif
-    fs->seek_pos = offs + len;
+    fs->seek_pos += len;
     buf->used = len;
 }
 
@@ -119,13 +121,16 @@ void    fs_read_random(FS_INFO *fs, char *buf, int len, OFF_T offs,
 	fprintf(logfp, "%s: read offs %.0f len %d (%s)\n",
 		myname, (double) offs, len, comment);
 
-    if (LSEEK(fs->fd, offs, SEEK_SET) != offs)
-		error("read random seek error: offset %llu: %m (%s)", 
-		  (ULLONG) offs, comment);
+    if (fs->seek_pos != offs) {
+    	if (LSEEK(fs->fd, offs, SEEK_SET) != offs)
+			error("read random seek error: offset %llu: %m (%s)", 
+			  (ULLONG) offs, comment);
+    	fs->seek_pos = offs;
+	}
 
     if ((count = read(fs->fd, buf, len)) != len)
 		error("read random read error (%d@%llu): %m (%s)", 
 		  len, (ULLONG) offs, comment);
 
-    fs->seek_pos = offs + len;
+    fs->seek_pos += len;
 }
