@@ -323,7 +323,7 @@ find_inode_ntfs_file(FS_INFO *fs, DADDR_T addr, char *buf,
 static u_int8_t
 find_inode(FS_INFO *fs, INUM_T inum, FS_INODE *fsi, int flags, char *ptr) 
 {
-	int myflags = (FS_FLAG_AONLY | FS_FLAG_CONT | 
+	int myflags = (FS_FLAG_AONLY | FS_FLAG_CONT | FS_FLAG_META | 
 	  FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_NOABORT);
 
 	curinode = inum;
