@@ -520,8 +520,11 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
  * This gives us an interface to call an action on data and not care if
  * it is resident or not.
  *
- * used flag values: FS_FLAG_AONLY, FS_FLAG_SLACK
- * unused flag values: FS_FLAG_{UN}ALLOC, FS_FLAG_META, FS_FLAG_CONT, 
+ * used flag values: FS_FLAG_AONLY, FS_FLAG_SLACK, FS_FLAG_ALLOC, FS_FLAG_UNALLOC
+ * unused flag values: FS_FLAG_META, FS_FLAG_CONT, 
+ *
+ * FS_FLAG_UNALLOC in this case means sparse entries (non-resident only)
+ *
  */
 static void
 ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data, 
@@ -595,8 +598,11 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 				 * the no abort flag is set */
 				if (addr > fs->last_block) {
 					if (flags & FS_FLAG_NOABORT) {
-						fprintf (stderr, "Invalid address in run (too large): %lu",
-						  (ULONG)addr);
+						if (verbose) {
+							fprintf (logfp,
+							  "Invalid address in run (too large): %lu",
+							  (ULONG)addr);
+						}	
 						return;
 					}
 					else {
@@ -621,12 +627,24 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 					bufsize = (int)fsize;
 
 				myflags = FS_FLAG_CONT;
-				if (WALK_STOP == action(fs, addr, buf, bufsize, myflags, ptr)) {
-					if ((flags & FS_FLAG_AONLY) == 0)
-						fs_buf_free(fs_buf);
-					return;
-				}
 
+				/* Only do sparse clusters if UNALLOC is set */
+				if ((fs_data_run->flags & FS_DATA_SPARSE) && 
+				  (flags & FS_FLAG_UNALLOC)) {
+					if (WALK_STOP == action(fs, addr, buf, bufsize, myflags | FS_FLAG_UNALLOC, ptr)) {
+						if ((flags & FS_FLAG_AONLY) == 0)
+							fs_buf_free(fs_buf);
+						return;
+					}
+				}
+				else if (((fs_data_run->flags & FS_DATA_SPARSE) == 0) && 
+				  (flags & FS_FLAG_ALLOC)) {
+					if (WALK_STOP == action(fs, addr, buf, bufsize, myflags | FS_FLAG_ALLOC, ptr)) {
+						if ((flags & FS_FLAG_AONLY) == 0)
+							fs_buf_free(fs_buf);
+						return;
+					}
+				}
 				
 				if ((u_int64_t)fs->block_size >= fsize)
 					break;
@@ -701,7 +719,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 
 			/* Copy the name into ASCII */
 			if (attr->nlen)
-				uni2ascii(&attr->c.r.name, attr->nlen, name, sizeof(name));
+				uni2ascii((char *)&attr->c.r.name, attr->nlen, name, sizeof(name));
 			/* Call the unnamed $Data attribute, $Data */
 			else if (type == NTFS_ATYPE_DATA)
 				strncpy(name, "$Data", NTFS_MAXNAMLEN+1);
@@ -730,7 +748,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 
 			/* Copy the name */
 			if (attr->nlen)
-				uni2ascii(&attr->c.nr.name, attr->nlen, name, sizeof(name));
+				uni2ascii((char *)&attr->c.nr.name, attr->nlen, name, sizeof(name));
 			else if (type == NTFS_ATYPE_DATA)
 				strncpy(name, "$Data", NTFS_MAXNAMLEN+1);
 			else
@@ -814,7 +832,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 				fs_name->next = NULL;
 			}
 
-			uni2ascii(&fname->name, fname->nlen, fs_name->name, 256);
+			uni2ascii((char *)&fname->name, fname->nlen, fs_name->name, 256);
 			fs_name->par_inode = getu48(fs, fname->par_ref);
 			fs_name->par_seq = getu16(fs, fname->par_seq);
 			
@@ -998,8 +1016,9 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 	act_buf = buf = mymalloc (act_bufleft);
 	endaddr = (DADDR_T)buf + fs_data_attrlist->size;
 
-	ntfs_data_walk(ntfs, fs_data_attrlist, (FS_FLAG_ALLOC | FS_FLAG_CONT), 
-	   buffer_action, "attrlist");
+	ntfs_data_walk(ntfs, fs_data_attrlist, 
+	  (FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_CONT), 
+	  buffer_action, "attrlist");
 
 	/* this value should be zero, if not then we didn't read all of the
 	 * buffer
@@ -1232,7 +1251,7 @@ ntfs_load_attrdef (NTFS_INFO *ntfs)
     ntfs->attrdef = (ntfs_attrdef *)act_buf;
     
     ntfs_data_walk(ntfs, fs_data,
-      FS_FLAG_ALLOC | FS_FLAG_CONT, buffer_action, "");
+      FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_CONT, buffer_action, "");
 
     if (act_bufleft)
         error ("space still left after walking $Attr data");
@@ -1259,7 +1278,7 @@ ntfs_attrname_lookup(FS_INFO *fs, u_int16_t type, char *name, int len)
     
     while (getu32(fs, attrdef->type)) {
 		if (getu32(fs, attrdef->type) == type) {
-        	uni2ascii(attrdef->label, 128, name, len);
+        	uni2ascii((char *)attrdef->label, 128, name, len);
 			return;
 		}
         attrdef++;
@@ -1571,7 +1590,7 @@ ntfs_fsstat(FS_INFO *fs, FILE *hFile)
 		error ("Volume Name attribute not found in $Volume");
 	
 	if ((fs_data->flags & FS_DATA_RES) && (fs_data->size)) {
-		uni2ascii(fs_data->buf, fs_data->size, asc, 128);
+		uni2ascii((char *)fs_data->buf, fs_data->size, asc, 128);
 		fprintf(hFile, "Volume Name: %s\n", asc);
 	}
 
@@ -1617,7 +1636,7 @@ ntfs_fsstat(FS_INFO *fs, FILE *hFile)
 	attrdeftmp = ntfs->attrdef;
 
 	while (getu32(fs, attrdeftmp->type)) {
-		uni2ascii(attrdeftmp->label, 128, asc, 128);
+		uni2ascii((char *)attrdeftmp->label, 128, asc, 128);
 		printf("%s: %d\n", asc, getu32(fs, attrdeftmp->type));
 
 		attrdeftmp++;
@@ -1668,7 +1687,7 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 
     print_dosmode(hFile, fs_inode->mode);
    
-    fprintf(hFile, "Size: %lu\n", (ULONG) fs_inode->size);
+    fprintf(hFile, "Size: %llu\n", (ULLONG) fs_inode->size);
     fprintf(hFile, "Links: %lu\n", (ULONG) fs_inode->nlink);
 
     if (fs_inode->name) {
@@ -1771,9 +1790,9 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
     
         ntfs_attrname_lookup(fs, fs_data->type, type, 256);
 
-        printf("Type: %s (%i-%i)   Name: %s   %sResident   size: %d\n",
+        printf("Type: %s (%i-%i)   Name: %s   %sResident   size: %llu\n",
           type, fs_data->type, fs_data->id, fs_data->name,
-          (fs_data->flags & FS_DATA_NONRES)?"Non-":"", (int)fs_data->size);
+          (fs_data->flags & FS_DATA_NONRES)?"Non-":"", (ULLONG)fs_data->size);
     
         if (fs_data->flags & FS_DATA_NONRES) {
             printidx = 0;
